package com.chelvc.framework.common.serializer;

import java.io.IOException;
import java.util.Map;

import com.chelvc.framework.common.util.ObjectUtils;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;
import com.fasterxml.jackson.databind.deser.ContextualDeserializer;
import com.fasterxml.jackson.databind.deser.std.StdDeserializer;

/**
 * JSON枚举反序列化处理器
 *
 * @author Woody
 * @date 2024/1/30
 */
public class JacksonEnumerateDeserializer extends StdDeserializer<Object> implements ContextualDeserializer {
    private Class<?> target;

    public JacksonEnumerateDeserializer() {
        super(Object.class);
    }

    @Override
    @SuppressWarnings({"unchecked", "rawtypes"})
    public Object deserialize(JsonParser parser, DeserializationContext context) throws IOException {
        Class<?> clazz = ObjectUtils.ifNull(this.target, Object.class);
        if (Enum.class.isAssignableFrom(clazz)) {
            Object value = parser.readValueAs(Object.class);
            if (value instanceof Map) {
                value = ((Map<?, ?>) value).keySet().stream().findFirst().orElse(null);
            }
            return Enum.valueOf((Class<Enum>) clazz, String.valueOf(value));
        }
        return parser.readValueAs(clazz);
    }

    @Override
    public JsonDeserializer<?> createContextual(DeserializationContext context, BeanProperty property) {
        this.target = context.getContextualType().getRawClass();
        return this;
    }
}
