package com.chelvc.framework.database.context;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import com.baomidou.mybatisplus.core.conditions.AbstractWrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.support.ColumnCache;
import com.baomidou.mybatisplus.core.toolkit.support.SFunction;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import lombok.Getter;
import lombok.NonNull;

/**
 * 对象实体Getter方法上下文
 *
 * @author Woody
 * @date 2024/1/30
 */
@Getter
public class GetterContext<T, R> implements SFunction<T, R> {
    /**
     * 目标属性
     */
    private final String property;

    /**
     * 方法被引用对象
     */
    private final Class<?> reference;

    /**
     * 属性直接所属对象
     */
    private final Class<?> declaring;

    public GetterContext(@NonNull String property, @NonNull Class<?> reference, @NonNull Class<?> declaring) {
        this.property = property;
        this.reference = reference;
        this.declaring = declaring;
    }

    @Override
    @SuppressWarnings("unchecked")
    public R apply(T t) {
        Method getter = ObjectUtils.lookupPropertyGetter(this.declaring, this.property);
        AssertUtils.nonnull(getter,
                () -> "Getter method not found: " + this.declaring.getName() + "." + this.property);
        try {
            return (R) getter.invoke(t);
        } catch (IllegalAccessException | InvocationTargetException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 自定义Getter方法引用查询包装器
     */
    public static class Query extends LambdaQueryWrapper<Object> {
        private static final Field PARAM_NAME_SEQ_FIELD = ObjectUtils.getField(AbstractWrapper.class, "paramNameSeq");
        private static final Field PARAM_NAME_VALUE_PAIRS_FIELD =
                ObjectUtils.getField(AbstractWrapper.class, "paramNameValuePairs");

        public Query(Class<Object> clazz) {
            super(clazz);
        }

        @Override
        protected LambdaQueryWrapper<Object> instance() {
            LambdaQueryWrapper<Object> instance = new Query(this.getEntityClass());
            // 复制paramNameSeq、paramNameValuePairs，不然会出现子查询参数为空的情况
            ObjectUtils.setValue(instance, PARAM_NAME_SEQ_FIELD, this.paramNameSeq);
            ObjectUtils.setValue(instance, PARAM_NAME_VALUE_PAIRS_FIELD, this.paramNameValuePairs);
            return instance;
        }

        @Override
        @SuppressWarnings("unchecked")
        protected String columnToString(SFunction<Object, ?> getter, boolean onlyColumn) {
            ColumnCache column = DatabaseContextHolder.getter2column((GetterContext<Object, ?>) getter);
            return onlyColumn ? column.getColumn() : column.getColumnSelect();
        }
    }
}
